/* 
    ======================================================================================= 
     "This sample program is provided AS IS and may be used, executed, copied and modified
     without royalty payment by customer

     (a) for its own instruction and study,
     (b) in order to develop applications designed to run with an IBM WebSphere product,
         either for customer's own internal use or for redistribution by customer, as part
         of such an application, in customer's own products."

     Product 5655-FLW,  (C) COPYRIGHT International Business Machines Corp., 2006, 2010
     All Rights Reserved * Licensed Materials - Property of IBM
    ======================================================================================= 
 */
package bpc.samples;

import java.rmi.RemoteException;

import javax.ejb.CreateException;
import javax.faces.context.FacesContext;
import javax.el.ValueExpression;
import javax.naming.NamingException;

import com.ibm.bpc.clientcore.exception.CommunicationException;
import com.ibm.bpe.api.BusinessFlowManagerService;
import com.ibm.bpe.clientmodel.BFMConnection;
import com.ibm.task.api.HumanTaskManagerService;
import com.ibm.task.clientmodel.HTMConnection;

/**
 * The singleton ServiceFactory manages the references for different Business
 * Process Choreographer modules. <BR>
 * To configure the factory, add a managed bean in one of the JSF configuration
 * files. The managed bean must be named "bfmConnection" and "htmConnection" and
 * have application scope.
 * 
 * Sample configuration in a faces-config.xml file:
 * 
 * <pre>
 * 
 * 	 &lt;managed-bean&gt;
 *  &lt;managed-bean-name&gt;ServiceFactory&lt;/managed-bean-name&gt;
 *  &lt;managed-bean-class&gt;bpc.samples.ServiceFactory&lt;/managed-bean-class&gt;
 *  &lt;managed-bean-scope&gt;application&lt;/managed-bean-scope&gt;
 *  
 *  &lt;managed-property&gt;
 *  &lt;property-name&gt;bfmConnection&lt;/property-name&gt;
 *  &lt;value&gt;#{bfmConnection}&lt;/value&gt;
 *  &lt;/managed-property&gt;
 *  
 *  &lt;managed-property&gt;
 *  &lt;property-name&gt;htmConnection&lt;/property-name&gt;
 *  &lt;value&gt;#{htmConnection}&lt;/value&gt;
 *  &lt;/managed-property&gt;
 *  
 *  &lt;/managed-bean&gt;
 *  
 *  &lt;managed-bean&gt;
 *  &lt;managed-bean-name&gt;bfmConnection&lt;/managed-bean-name&gt;
 *  &lt;managed-bean-class&gt;com.ibm.bpe.clientmodel.BFMConnection&lt;/managed-bean-class&gt;
 *  &lt;managed-bean-scope&gt;application&lt;/managed-bean-scope&gt;
 *  &lt;managed-property&gt;
 *  &lt;property-name&gt;jndiName&lt;/property-name&gt;
 *  &lt;value&gt;java:comp/env/ejb/LocalBusinessProcessHome&lt;/value&gt;
 *  &lt;/managed-property&gt;
 *  &lt;/managed-bean&gt;
 *  
 *  &lt;managed-bean&gt;
 *  &lt;managed-bean-name&gt;htmConnection&lt;/managed-bean-name&gt;
 *  &lt;managed-bean-class&gt;com.ibm.task.clientmodel.HTMConnection&lt;/managed-bean-class&gt;
 *  &lt;managed-bean-scope&gt;application&lt;/managed-bean-scope&gt;
 *  &lt;managed-property&gt;
 *  &lt;property-name&gt;jndiName&lt;/property-name&gt;
 *  &lt;value&gt;java:comp/env/ejb/LocalHumanTaskManagerEJB&lt;/value&gt;
 *  &lt;/managed-property&gt;
 *  &lt;/managed-bean&gt;
 * 
 * 
 * </pre>
 * 
 * <BR>
 */
public class ServiceFactory {

	/* -- Class variables and methods -- */
	static private ServiceFactory instance = null;

	/**
	 * Get the singleton instance
	 * 
	 * @return the singleton
	 */
	static public ServiceFactory getInstance() {

		if (instance == null) {
			// Make sure that a managed bean with the name ServiceFactory is
			// configured in the faces-config.xml
			//
			FacesContext ctx = FacesContext.getCurrentInstance();
			ValueExpression exp = ctx.getApplication().getExpressionFactory()
					.createValueExpression(ctx.getELContext(),
							"#{ServiceFactory}", ServiceFactory.class);
			instance = (ServiceFactory) exp.getValue(ctx.getELContext());
		}

		return instance;
	}

	/* -- Instance variables and methods -- */
	private BFMConnection bfmConnection;
	private HTMConnection htmConnection;

	/**
	 * <p>
	 * Default C'tor
	 * </p>
	 */
	public ServiceFactory() {
	}

	/**
	 * Returns the underlying connection object
	 * 
	 * @return the Business Flow Manager connection
	 */
	public BFMConnection getBfmConnection() {
		return bfmConnection;
	}

	/**
	 * Sets the connection object to determin the service
	 * 
	 * @param bfmConnection
	 *            : connection which determines the BusinessFlowManagerService
	 */
	public void setBfmConnection(BFMConnection bfmConnection) {
		this.bfmConnection = bfmConnection;
	}

	/**
	 * Returns the underlying connection object
	 * 
	 * @return the Human Task Manager connection
	 */
	public HTMConnection getHtmConnection() {
		return htmConnection;
	}

	/**
	 * Sets the connection object to determine the service
	 * 
	 * @param htmConnection
	 *            : the connection which determines the HumanTaskManageService
	 */
	public void setHtmConnection(HTMConnection htmConnection) {
		this.htmConnection = htmConnection;
	}

	/**
	 * Returns the Business Flow Manage Service.
	 * 
	 * @return The BFM service API
	 * @throws RemoteException
	 * @throws CreateException
	 * @throws NamingException
	 */
	public BusinessFlowManagerService getBusinessFlowManagerService()
			throws RemoteException, CreateException, NamingException,
			CommunicationException {

		return bfmConnection.getBusinessFlowManagerService();
	}

	/**
	 * Returns the Human Task Manager Service.
	 * 
	 * @return The HTM service API
	 * @throws RemoteException
	 * @throws CreateException
	 * @throws NamingException
	 */
	public HumanTaskManagerService getHumanTaskManagerService()
			throws RemoteException, CreateException, NamingException,
			CommunicationException {

		return htmConnection.getHumanTaskManagerService();
	}
}
