/* 
    ======================================================================================= 
     "This sample program is provided AS IS and may be used, executed, copied and modified
     without royalty payment by customer

     (a) for its own instruction and study,
     (b) in order to develop applications designed to run with an IBM WebSphere product,
         either for customer's own internal use or for redistribution by customer, as part
         of such an application, in customer's own products."

     Product 5655-FLW,  (C) COPYRIGHT International Business Machines Corp., 2006, 2010
     All Rights Reserved * Licensed Materials - Property of IBM
    ======================================================================================= 
*/

package bpc.samples.bean;

import java.util.logging.Logger;

import bpc.samples.ServiceFactory;

import com.ibm.bpe.api.BusinessFlowManagerService;
import com.ibm.bpe.api.OID;
import com.ibm.bpe.api.PIID;
import com.ibm.task.api.QueryResultSet;
import com.ibm.task.api.Task;
import com.ibm.task.clientmodel.HTMConnection;
import com.ibm.task.clientmodel.bean.TaskInstanceBean;

/**
 * <code>CustomTaskInstanceBean extends a <code>com.ibm.task.clientmodel.bean.TaskInstanceBean<code>
 * and adds the following custom properties to the bean's properties:
 * <ul>
 * 	<li>"department"  
 * 	<li>"orderNo" 
 * </ul>
 */
public class CustomTaskInstanceBean extends TaskInstanceBean {

	public static final long serialVersionUID = 1L;

	Logger logger = Logger.getLogger("CustomClient");

	/* -- Static properties -- */
	public static String PROPERTY_NAME_DEPARTMENT = "department";

	public static String PROPERTY_NAME_ORDER_NO = "orderNo";

	private static String DEFAULT_VALUE = "Not set";

	/* -- Instance variables -- */
	private String department = DEFAULT_VALUE;

	private String orderNo = DEFAULT_VALUE;

	/**
	 * Constructs a custom task instance bean from a task instance returned by the API.
	 * 
	 * @param taskInstance  task instance as returned from the human task manager API.
	 * @param htmConnection the connection to Human Task Manager
	 */
	public CustomTaskInstanceBean(Task taskInstance, HTMConnection htmConnection) {
		super(taskInstance, htmConnection);

		PIID piid = null;
		OID oid = taskInstance.getContainmentContextID();
		if (oid != null && oid instanceof PIID) {
			try {
				BusinessFlowManagerService bfm = ServiceFactory.getInstance().getBusinessFlowManagerService();
				piid = (PIID) oid;
				String valueDepartment = bfm.getCustomProperty(piid, PROPERTY_NAME_DEPARTMENT);
				if (valueDepartment != null) {
					department = valueDepartment;
				}
				String valueOrderNo = bfm.getCustomProperty(piid, PROPERTY_NAME_ORDER_NO);
				if (valueOrderNo != null) {
					orderNo = valueOrderNo;
				}
			} catch (Exception e) {
				logger.info("Error while accessing custom properties: "
						+ e.getMessage());
			}
		}
	}

	/**
	 * Constructs a custom process instance bean from a query result set
	 * returned by the BusinessFlowManager API
	 * 
	 * @param resultSet
	 *            the query result from a query call; represents a row of this
	 *            result
	 * @param bfmConnection
	 *            the connection to the Business Flow Manager
	 */
	public CustomTaskInstanceBean(QueryResultSet resultSet,
			HTMConnection htmConnection) {

		// Make sure the super class can read the properties from the resultSet
		//
		super(resultSet, htmConnection);

		// Then read the custom properties from the result set.
		// Iterate over the columns in the result set and check
		// if the custom properties were included.
		//
		for (int col = 1; col <= resultSet.numberColumns(); col++) {

			String table = resultSet.getTableDisplayName(col);
			String column = resultSet.getColumnDisplayName(col);
			logger.info("Analyzing: table=" + table + " column=" + column
					+ " value=" + resultSet.getObject(col));

			// Make sure that the select AND where clause contains the following
			// sequence of attributes:
			// ... PROCESS_ATTRIBUTE0.NAME="orderNo",
			// PROCESS_ATTRIBUTE1.NAME="department",....
			// 
			if (table.equalsIgnoreCase("PROCESS_ATTRIBUTE")
					&& column.equalsIgnoreCase("VALUE")) {

				if (orderNo.equalsIgnoreCase(DEFAULT_VALUE))
					orderNo = resultSet.getString(col);

				else if (department.equalsIgnoreCase(DEFAULT_VALUE))
					department = resultSet.getString(col);
			}
		}
	}

	public String getDepartment() {
		return department;
	}

	public void setDepartment(String department) {
		this.department = department;
	}

	public String getOrderNo() {
		return orderNo;
	}

	public void setOrderNo(String orderNo) {
		this.orderNo = orderNo;
	}

}
