/* 
======================================================================================= 
 "This sample program is provided AS IS and may be used, executed, copied and modified
 without royalty payment by customer

 (a) for its own instruction and study,
 (b) in order to develop applications designed to run with an IBM WebSphere product,
     either for customer's own internal use or for redistribution by customer, as part
     of such an application, in customer's own products."

 Product 5655-FLW,  (C) COPYRIGHT International Business Machines Corp., 2006, 2010
 All Rights Reserved * Licensed Materials - Property of IBM
======================================================================================= 
*/
package bpc.samples.command;

import java.util.Iterator;
import java.util.List;

import bpc.samples.CustomException;
import bpc.samples.ServiceFactory;

import com.ibm.bpc.clientcore.ClientException;
import com.ibm.bpc.clientcore.Command;
import com.ibm.bpe.api.BusinessFlowManagerService;
import com.ibm.bpe.api.ProcessInstanceData;
import com.ibm.bpe.clientmodel.bean.ProcessInstanceBean;

/**
 * DeleteProcessInstancesCommand will delete process instances
 * that are in a state of finished, failed or terminated. The 
 * process instances to be deleted are passed to the command as a
 * List of <code>com.ibm.bpe.clientmodel.bean.ProcessInstanceBean</code>.
 * 
 * If the System fails to delete a process instance, for example, 
 * a process in the running state an exception is loggedd.
 */
public class DeleteProcessInstancesCommand implements Command {

	/* (non-Javadoc)
	 * @see com.ibm.bpc.clientcore.Command#execute(java.util.List)
	 */
	public String execute(List selectedObjects) throws ClientException {
		
		try
		{
	        BusinessFlowManagerService bfm = ServiceFactory.getInstance().getBusinessFlowManagerService() ;
	        
	        for( Iterator iter = selectedObjects.iterator(); iter.hasNext(); ) {
	            Object obj = iter.next();
	            if( obj instanceof ProcessInstanceBean ) {
	            	ProcessInstanceBean bean = (ProcessInstanceBean) obj ;
	                
	                try {
		                
		            	bfm.delete(bean.getID());
	                }
	                catch( Exception e) {
	                	e.printStackTrace();
	                }
	            }
	        }
			
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new CustomException("Exception occured: " + e.getMessage());
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see com.ibm.bpc.clientcore.Command#isMultiSelectEnabled()
	 */
	public boolean isMultiSelectEnabled() {
		return true;
	}

	/* (non-Javadoc)
	 * @see com.ibm.bpc.clientcore.Command#isApplicable(java.util.List)
	 */
	public boolean[] isApplicable(List selectedObjects) {
		
		boolean[] applicable = null;
		
		if(selectedObjects!= null)
		{
			applicable = new boolean[selectedObjects.size()];
			int i = 0;
			
	        for( Iterator iter = selectedObjects.iterator(); iter.hasNext(); ) {
	            Object obj = iter.next();
	            if( obj instanceof ProcessInstanceBean ) {
	            	ProcessInstanceBean bean = (ProcessInstanceBean) obj ;
	            	
	            	int state = bean.getExecutionState();
	            	
	            	if(state == ProcessInstanceData.STATE_FINISHED || 
	                   state == ProcessInstanceData.STATE_FAILED ||
					   state == ProcessInstanceData.STATE_TERMINATED )
	            	{
	            		applicable[i] = true;
	            	}
	            	else
	            	{
		            	applicable[i] = false;
	            	}
	            }
	            else
	            {
	            	applicable[i] = false;
	            }
	            i++;
	        }
		}
		
		return applicable;
	}

	/* (non-Javadoc)
	 * @see com.ibm.bpc.clientcore.Command#setContext(java.lang.Object)
	 */
	public void setContext(Object arg0) {
		//not required
	}

}
