/* 
    ======================================================================================= 
     "This sample program is provided AS IS and may be used, executed, copied and modified
     without royalty payment by customer
  
     (a) for its own instruction and study,
     (b) in order to develop applications designed to run with an IBM WebSphere product,
         either for customer's own internal use or for redistribution by customer, as part
         of such an application, in customer's own products."

     Product 5655-FLW,  (C) COPYRIGHT International Business Machines Corp., 2006, 2010
     All Rights Reserved * Licensed Materials - Property of IBM
    ======================================================================================= 
*/

package bpc.samples.converter;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;

/**
 * Custom converter for converting generated task names to more meaningful GUI representations
 */
public class TaskNameConverter implements Converter {
	
	//model names as in the BPEL / TEL model
	private final static String TASK2_MODEL_NAME = "OrderManagementProcess$OrderManagementProcessTask2";
	private final static String TASK4_MODEL_NAME = "OrderManagementProcess$OrderManagementProcessTask4";
	private final static String TASK5_MODEL_NAME = "OrderManagementProcess$OrderManagementProcessTask5";

	//Define the label names for the task names above
	private final static String TASK2_LABEL = "Update order";
	private final static String TASK4_LABEL = "Decide";
	private final static String TASK5_LABEL = "Approve";

	/* 
	 * Convert diplay names (label names) of tasks to their model names
	 *  
	 * @see javax.faces.convert.Converter#getAsObject(javax.faces.context.FacesContext, javax.faces.component.UIComponent, java.lang.String)
	 */
	public Object getAsObject(FacesContext context, UIComponent uiComponent, String label) {
		
		String modelName = null;
		
		if(label != null)
		{
			if (label.equals(TASK2_LABEL))
			{
				modelName = TASK2_MODEL_NAME;
			}
			else if (label.equals(TASK4_LABEL))
			{
				modelName = TASK4_MODEL_NAME;
			}
			else if (label.equals(TASK5_LABEL))
			{
				modelName = TASK5_MODEL_NAME;
			}
		}
		
		return modelName;
	}

	/* 
	 * Return the label for the task model name
	 * 
	 * Example: If the model name of the task is : MyProcess$MyProcessTask1 this method returns
	 *          a more readable representation, e.g. Approve
	 * 
	 * @see javax.faces.convert.Converter#getAsString(javax.faces.context.FacesContext, javax.faces.component.UIComponent, java.lang.Object)
	 */
	public String getAsString(FacesContext context, UIComponent uiComponent, Object modelName) {
		
		String label = null;
		
		if(modelName != null)
		{
			if (modelName.equals(TASK2_MODEL_NAME))
			{
				label = TASK2_LABEL;
			}
			else if (modelName.equals(TASK4_MODEL_NAME))
			{
				label = TASK4_LABEL;
			}
			else if (modelName.equals(TASK5_MODEL_NAME))
			{
				label = TASK5_LABEL;
			}
			else
			{
				label = modelName.toString();
			}
		}
		
		return label;
	}

}
