/* 
    ======================================================================================= 
     "This sample program is provided AS IS and may be used, executed, copied and modified
     without royalty payment by customer
  
     (a) for its own instruction and study,
     (b) in order to develop applications designed to run with an IBM WebSphere product,
         either for customer's own internal use or for redistribution by customer, as part
         of such an application, in customer's own products."

     Product 5655-FLW,  (C) COPYRIGHT International Business Machines Corp., 2006, 2010
     All Rights Reserved * Licensed Materials - Property of IBM
    ======================================================================================= 
*/

package bpc.samples.converter;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;

import com.ibm.task.api.Task;

/**
 * Converts the state information of tasks to a meaningful string.
 * 
 */
public class TaskStateConverter implements Converter {
	
	private static final String LABEL_READY           = "ready";
	private static final String LABEL_CLAIMED         = "claimed";
	private static final String LABEL_EXPIRED         = "expired";
	private static final String LABEL_FAILED          = "failed";
	private static final String LABEL_FINISHED        = "finished";
	private static final String LABEL_FORWARDED       = "forwarded";
	private static final String LABEL_INACTIVE        = "inactive";
	private static final String LABEL_RUNNING         = "running";
	private static final String LABEL_TERMINATED      = "terminated";

	private static final String LABEL_UNKNOWN = "unknown";

	/* (non-Javadoc)
	 * @see javax.faces.convert.Converter#getAsObject(javax.faces.context.FacesContext, javax.faces.component.UIComponent, java.lang.String)
	 */
	public Object getAsObject(FacesContext context, UIComponent component, String label) {
		
		int state = Task.STATE_INACTIVE;
		
		if(label != null)
		{
			if(label.equals(LABEL_READY))
			{
				state = Task.STATE_READY;
			}
			else if(label.equals(LABEL_CLAIMED))
			{
				state = Task.STATE_CLAIMED;
			}
			else if(label.equals(LABEL_EXPIRED))
			{
				state = Task.STATE_EXPIRED;
			}
			else if(label.equals(LABEL_FAILED))
			{
				state = Task.STATE_FAILED;
			}
			else if(label.equals(LABEL_FINISHED))
			{
				state = Task.STATE_FINISHED;
			}
			else if(label.equals(LABEL_FORWARDED))
			{
				state = Task.STATE_FORWARDED;
			}
			else if(label.equals(LABEL_INACTIVE))
			{
				state = Task.STATE_INACTIVE;
			}
			else if(label.equals(LABEL_RUNNING))
			{
				state = Task.STATE_RUNNING;
			}
			else if(label.equals(LABEL_TERMINATED))
			{
				state = Task.STATE_TERMINATED;
			}
		}
		
		return new Integer(state);
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.Converter#getAsString(javax.faces.context.FacesContext, javax.faces.component.UIComponent, java.lang.Object)
	 */
	public String getAsString(FacesContext context, UIComponent component, Object stateObject) {
		
		String label = LABEL_UNKNOWN;
		
		if(stateObject != null && stateObject instanceof Integer)
		{
			int state = ((Integer)stateObject).intValue();
			
			switch(state)
			{
		        case Task.STATE_READY:
		    	    label = LABEL_READY;
		    	    break;
		        case Task.STATE_CLAIMED:
		    	    label = LABEL_CLAIMED;
		    	    break;
		        case Task.STATE_RUNNING:
		    	    label = LABEL_RUNNING;
		    	    break;
		        case Task.STATE_FINISHED:
		    	    label = LABEL_FINISHED;
		    	    break;
		        case Task.STATE_FAILED:
		    	    label = LABEL_FAILED;
		    	    break;
		        case Task.STATE_TERMINATED:
		    	    label = LABEL_TERMINATED;
		    	    break;
		        case Task.STATE_EXPIRED:
		    	    label = LABEL_EXPIRED;
		    	    break;
		        case Task.STATE_FORWARDED:
		    	    label = LABEL_FORWARDED;
		    	    break;
		        case Task.STATE_INACTIVE:
		    	    label = LABEL_INACTIVE;
		    	    break;
			    default: label = LABEL_UNKNOWN;
			}
		}
		return label;
	}

}
